/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2017 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    usb_compiler.h
 * Purpose: USB Library Compiler specific definitions
 *----------------------------------------------------------------------------*/

#ifndef __USB_COMPILER_H__
#define __USB_COMPILER_H__

#include "stdint.h"

#if   defined ( __CC_ARM )                                            /* ARM Compiler 4/5 */
#ifndef   __ASM
  #define __ASM                     __asm
#endif
#ifndef   __REV
  #define __REV                     __rev
#endif
#ifndef   __NOP
  #define __NOP                     __nop
#endif
#ifndef   __INLINE
  #define __INLINE                  __inline
#endif
#ifndef   __STATIC_INLINE
  #define __STATIC_INLINE           static __inline
#endif
#ifndef   __NO_RETURN
  #define __NO_RETURN               __declspec(noreturn)
#endif
#ifndef   __USED
  #define __USED                    __attribute__((used))
#endif
#ifndef   __WEAK
  #define __WEAK                    __attribute__((weak))
#endif
#undef    __PACKED
#ifndef   __PACKED
  #define __PACKED                  __attribute__((packed))
#endif
#ifndef   __ALIGNED
  #define __ALIGNED(x)              __attribute__((aligned(x)))
#endif
#ifndef   __UNALIGNED_UINT32
  #define __UNALIGNED_UINT32(x)     (*((__packed uint32_t *)(x)))
#endif
#ifndef   __AT_ADDR
  #define __AT_ADDR(x)              __attribute__((at(x)))
#endif
#ifndef   __SECTION__
  #define __SECTION__(x)            __attribute__((section(#x)))
#endif
#ifndef   __SECTION
  #define __SECTION(x)              __SECTION__(x)
#endif

#elif defined (__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)       /* ARM Compiler 6 */
#ifndef   __ASM
  #define __ASM                     __asm
#endif
#ifndef   __REV
  #define __REV                     __builtin_bswap32
#endif
#ifndef   __NOP
  #define __NOP                     __builtin_arm_nop
#endif
#ifndef   __INLINE
  #define __INLINE                  __inline
#endif
#ifndef   __STATIC_INLINE
  #define __STATIC_INLINE           static __inline
#endif
#ifndef   __NO_RETURN
  #define __NO_RETURN               __attribute__((noreturn))
#endif
#ifndef   __USED
  #define __USED                    __attribute__((used))
#endif
#ifndef   __WEAK
  #define __WEAK                    __attribute__((weak))
#endif
#ifndef   __PACKED
  #define __PACKED                  __attribute__((packed, aligned(1)))
#endif
#ifndef   __ALIGNED
  #define __ALIGNED(x)              __attribute__((aligned(x)))
#endif
#ifndef   __UNALIGNED_UINT32
  #pragma clang diagnostic push
  #pragma clang diagnostic ignored "-Wpacked"
  struct __attribute__((packed))    T_UINT32 { uint32_t v; };
  #pragma clang diagnostic pop
  #define __UNALIGNED_UINT32(x)     (((struct T_UINT32 *)(x))->v)
#endif
#ifndef   __AT_ADDR
  #define __AT_ADDR__(x)            __attribute__((section(".ARM.__AT_"#x)))
  #define __AT_ADDR(x)              __AT_ADDR__(x)
#endif
#ifndef   __SECTION
  #define __SECTION__(x)            __attribute__((section(#x)))
  #define __SECTION(x)              __SECTION__(x)
#endif

#else
  #error Unsupported compiler
#endif

#endif  // __USB_COMPILER_H__
